"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const loader_1 = require("../loader");
const error_1 = require("../error");
const helpers_1 = require("../helpers");
/**
 * Print help info.
 * @param brand brand name
 * @param cmd command
 */
const outputHelp = (brand, cmd) => {
    if (cmd.description) {
        helpers_1.logger.info(cmd.description);
        helpers_1.logger.newline();
    }
    helpers_1.logger.info('Usage:');
    const usage = cmd.usage || `${cmd.name} [options]`;
    helpers_1.logger.info(helpers_1.logger.indent(`$ ${brand} ${usage}`));
    // output commands if default command
    if (cmd.name === 'default') {
        const userCmds = [...new Set(Object.values(loader_1.userCommands))];
        const coreCmds = [...new Set(Object.values(loader_1.coreCommands))];
        if (userCmds.length + coreCmds.length) {
            helpers_1.logger.newline();
            helpers_1.logger.info('Commands:');
            const commandsObj = userCmds
                .concat(coreCmds)
                .filter(i => !i.hidden && i.name !== 'default')
                .reduce((obj, item) => {
                const key = `${item.name}${item.alias ? `(${item.alias})` : ''}`;
                const value = item.description || '-';
                return Object.assign({}, obj, { [key]: value });
            }, {});
            helpers_1.logger.info(helpers_1.logger.indent(helpers_1.logger.table(commandsObj)));
        }
    }
    if (cmd.options) {
        helpers_1.logger.newline();
        helpers_1.logger.info('Options:');
        const options = cmd.options;
        const optionsObj = Object.keys(options).reduce((o, i) => {
            const opt = options[i];
            let key = `--${i}`;
            let value = '-';
            if (typeof opt !== 'string') {
                key = `--${i}${opt.alias ? `, -${opt.alias}` : ''}`;
                value = opt['description'] || '-';
            }
            return Object.assign({}, o, { [key]: value });
        }, {});
        helpers_1.logger.info(helpers_1.logger.indent(helpers_1.logger.table(optionsObj)));
    }
    if (cmd.examples) {
        helpers_1.logger.newline();
        helpers_1.logger.info('Examples:');
        if (typeof cmd.examples === 'string') {
            helpers_1.logger.info(helpers_1.logger.indent(`${cmd.examples}`));
        }
        else {
            helpers_1.logger.info(helpers_1.logger.indent(cmd.examples.join('\n')));
        }
    }
    if (cmd.suggestions) {
        helpers_1.logger.newline();
        helpers_1.logger.info('Suggestions:');
        if (typeof cmd.suggestions === 'string') {
            helpers_1.logger.info(helpers_1.logger.indent(`${cmd.suggestions}`));
        }
        else {
            helpers_1.logger.info(helpers_1.logger.indent(cmd.suggestions.join('\n')));
        }
    }
};
/**
 * Show sub command help.
 * @param name command name
 * @param ctx cli context
 */
const subCommandHelp = (name, ctx) => {
    const cmd = loader_1.userCommands[name];
    if (!cmd)
        return error_1.unknownCommand(name, `${ctx.brand} --help`);
    // custom help
    if (cmd.help) {
        if (typeof cmd.help === 'string') {
            return helpers_1.logger.info(cmd.help);
        }
        return cmd.help(ctx);
    }
    // default help
    outputHelp(ctx.brand, cmd);
};
const command = {
    name: 'help',
    usage: 'help <command>',
    description: 'output usage information',
    hidden: false,
    action: (ctx) => __awaiter(this, void 0, void 0, function* () {
        if (ctx.primary && ctx.primary !== 'help') {
            subCommandHelp(ctx.primary, ctx);
        }
        else if (ctx.primary === 'help' && ctx.secondary) {
            subCommandHelp(ctx.secondary, ctx);
        }
        else {
            outputHelp(ctx.brand, loader_1.userCommands.default || loader_1.coreCommands.default);
        }
        process.exit();
    })
};
exports.default = command;
