"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const fs_1 = require("fs");
const cache = {};
/**
 * Load dir commands
 * @param dir dir to load
 */
const loadCommands = (dir) => {
    const commandDir = path_1.join(__dirname, dir);
    // return if cached
    if (cache[dir])
        return cache[commandDir];
    // scanning commands
    cache[commandDir] = fs_1.readdirSync(commandDir).reduce((commands, item) => {
        try {
            const command = require(path_1.join(commandDir, item)).default;
            commands[command.name] = command;
            if (typeof command.alias === 'string') {
                commands[command.alias] = command;
            }
            else if (command.alias) {
                command.alias.forEach(a => {
                    commands[a] = command;
                });
            }
        }
        catch (e) { }
        return commands;
    }, {});
    return cache[commandDir];
};
/**
 * All core commands
 */
exports.coreCommands = loadCommands('./commands');
/**
 * All commands
 */
exports.userCommands = loadCommands('../commands');
/**
 * Load command by name
 * @param name command name
 */
exports.load = (name) => __awaiter(this, void 0, void 0, function* () {
    // help command
    if (['help', '--help', '-h'].includes(name)) {
        return exports.coreCommands.help;
    }
    // version command
    if (['version', '--version', '-V'].includes(name)) {
        return exports.coreCommands.version;
    }
    // try to load user command
    if (exports.userCommands[name]) {
        return exports.userCommands[name];
    }
    // try to load user default command
    if (exports.userCommands.default) {
        return exports.userCommands.default;
    }
    // fallback default command
    return exports.coreCommands.default;
});
