"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable @typescript-eslint/no-explicit-any */
const readline = require("readline");
const chalk_1 = require("chalk");
const ora_1 = require("ora");
// disable colors output for testing
chalk_1.default.enabled = process.env.NODE_ENV !== 'test';
/**
 * Writes a normal message.
 * @param message The message to show.
 */
exports.log = (message, ...optionalParams) => {
    console.log(message, ...optionalParams);
};
/**
 * Writes a normal information message.
 * This is the default type you should use.
 * @param message The message to show.
 */
exports.info = (message, ...optionalParams) => {
    exports.log(chalk_1.default.reset(message), ...optionalParams);
};
/**
 * Writes a warning message.
 * This is when the user might not be getting what they're expecting.
 * @param message The message to show.
 */
exports.warn = (message, ...optionalParams) => {
    exports.log(chalk_1.default.yellow(message), ...optionalParams);
};
/**
 * Writes an error message.
 * This is when something horribly goes wrong.
 * @param message The message to show.
 */
exports.error = (message, ...optionalParams) => {
    exports.log(chalk_1.default.red(message), ...optionalParams);
};
/**
 * Print a blank line.
 */
exports.newline = () => {
    exports.log();
};
/**
 * Prints a divider line
 */
exports.divider = () => {
    console.log(chalk_1.default.gray('--------------------------------------------------------------------------------'));
};
/**
 * Writes a debug message.
 * This is for devs only.
 * @param message The message to show.
 */
exports.debug = (message, title = 'DEBUG') => {
    exports.log(chalk_1.default.magenta(`↓↓↓ ----------[ ${title} ]---------- ↓↓↓`));
    exports.log(message);
    exports.log(chalk_1.default.magenta(`↑↑↑ ----------[ ${title} ]---------- ↑↑↑`));
};
/**
 * Creates a spinner and starts it up.
 * @param options The text for the spinner or an ora options.
 * @returns The Ora spinner.
 */
exports.spin = (options = '') => {
    return ora_1.default(options).start();
};
/**
 * Clear console.
 * @param title Default title
 */
exports.clear = (title) => {
    if (!process.stdout.isTTY)
        return;
    const blank = '\n'.repeat(process.stdout.rows || 30);
    exports.log(blank);
    readline.cursorTo(process.stdout, 0, 0);
    readline.clearScreenDown(process.stdout);
    title && exports.log(title);
};
